# bibtex_lib.awk
#
# Escape TeX-special constructions and convert HTML-constructs to LaTeX.
# General functions dealing with BibTeX entries.
#
# Notes:
# * This is an awk library.
#
# Programmed with gawk 3.1.1
#
# Copyright (C) by Volker Kuhlmann <VolkerKuhlmann@gmx.de>
# Released under the terms of the GNU General Public License (GPL) Version 2.
# See http://www.gnu.org/ for details.
#
# Volker Kuhlmann
#  16, 17, 18, 19, 20 Feb; 1, 2, 21 Mar; 29 Apr 2003
#

## Remove given fields from entry if they're empty
# (Fields are passed as space-separated field names in one only string)
function del_fields_if_empty(entry, fields           ,f,n) {
    split (fields, f)
    for (n in f) {
	if (entry[f[n]] == "") { delete entry[f[n]] }
    }
}

## Append a string to a field.
# Append sepstr first if field is non-empty; sepstr defaults to " ".
function addtofield(entry, field, addstr, sepstr) {
    if (sepstr == "") sepstr = " "
    if (entry[field] == "")
    	entry[field] = addstr;
    else
    	entry[field] = entry[field] sepstr addstr
}

## Remove leading white space
function delleadspc(str) {
    sub("^[ \t]*", "", str)
    return str
}

## Remove leading and trailing white space
function delspc(str) {
    sub("^[ \t]*", "", str)
    sub("[ \t]*$", "", str)
    return str
}

## Deal with author field
function rearrange_authors(entry, field,      au, a, n, ret) {
    a = split(entry[field], au, " *; *")
    ret = split_author(au[1])
    for (n=2; n <= a; n++) { ret = ret " and " split_author(au[n]) }
    entry[field] = ret
}
function split_author(author,        au, a, n, ret) {
    a = split(author, au, " *, *")
    if (a == 0) { return "" }
    if (a == 1) { return au[1] }
    ret = au[2] " " au[1]
    for (n=3; n <= a; n++) { ret = ret ", " au[n] }
    return ret
}

## Deal with title fields
# (Enclose words of only-uppercase characters in {})
# (Fields are passed as space-separated names in one only string.)
function escape_title(entry, fields               ,f,n) {
    split (fields, f)
    for (n in f) {
	if ( ! (f[n] in entry) ) continue
	entry[f[n]] = gensub ("([A-Z][A-Z0-9]+)", "{\\1}", "g", entry[f[n]])
	escape_words(entry, f[n])
    }
}
function escape_words(e,f) {
    e[f] = gensub ( \
	"(^|[- /,;:\\.?!\\[\\(])" "(C|C\\+\\+)" "([- /,;:\\.?!\\]\\)]|$)", \
	"\\1{\\2}\\3", "g", e[f])
}
#
#{ wordseps="-,/?!;:'@=+<>\\[\\]\\.\\(\\)\\|0-9" }
function escape_title_try1(entry, field,             words, w, n) {
    if ( ! (field in entry) ) return
    w = split(entry[field], words, "[ \\t\\n\\r]*")
    for (n=1; n <= w; n++) {
    	#printf "++ word: '" words[n] "'"
	words[n] = gensub (\
	    "(["wordseps"]*)([A-Z0-9]*[A-Z][A-Z0-9]+)(["wordseps"]*)", \
	    "\\1{\\2}\\3", "", words[n])
    	#printf "    -> '" words[n] "'\n"
    }
    entry[field] = words[1]
    for (n=2; n <= w; n++) entry[field] = entry[field] " " words[n]
}
#{ wordseps="-,/?!;:'@=+<>\\[\\]\\.\\(\\)\\| \\t\\n\\r" }
function escape_title_try2(entry, field,             words, w, n) {
    if ( ! (field in entry) ) return
	entry[field] = gensub ("(["wordseps"])([A-Z][A-Z0-9]+)(["wordseps"])", \
			    "\\1{\\2}\\3", "g", entry[field])
    	#printf "    -> '" words[n] "'\n"
}

## Deal with TeX's special characters in the given fields.
# (Fields are passed as space-separated names in one only string.)
# Note: correction of space between numbers and units doesn't always work
# Note: only some of the HTML special character names are implemented
# Note: this will save some, but not all, conversion work
function escape_tex(entry, fields   	    ,f,ff) {
    split(fields, ff, " ")
    for (f in ff) {
    	## backslash, ampersand, and a few others
	gsub("\\\\", "\\backslash", entry[ff[f]]) # can't put \backslash{} here
	gsub("[{}^_%]", "\\\\&{}", entry[ff[f]])
    	gsub("~", "\\ensuremath{\\approx}", entry[ff[f]])
	entry[ff[f]] = translate_html(entry[ff[f]])
	gsub("[<>]", "\\ensuremath{&}", entry[ff[f]])
	
	## units
	# HZ
	entry[ff[f]] = gensub("([0-9]) *([pnmckMGT])Hz",
	    	    	    	"\\1\\\\,\\2Hz", "g", entry[ff[f]])
	entry[ff[f]] = gensub("([0-9]) *Hz", "\\1\\\\,Hz", "g", entry[ff[f]])
	# dB
	entry[ff[f]] = gensub("([0-9]) *dB",
	    	    	    	"\\1\\\\,dB", "g", entry[ff[f]])
	# W, A, V, m, s
	entry[ff[f]] = gensub("([0-9]) *([pnmckMGT])([WAVms])([.,?!)}/ ])",
	    	    	    	"\\1\\\\,\\2\\3\\4", "g", entry[ff[f]])
	entry[ff[f]] = gensub("([0-9]) *([WAVms])([.,?!)}/ ])",
	    	    	    	"\\1\\\\,\\2\\3", "g", entry[ff[f]])
    }
}

## Deal with some HTML-encodings
function translate_html(string) {
    	#gsub("&mu;", "\\ensuremath{\\mu}", string)
    	gsub(" *&mu; *", "\\muunit{}", string)
    	gsub("&Omega;", "\\ensuremath{\\Omega}", string)
	gsub("&deg;C", "\\celsius{}", string)
	gsub("&deg;", "\\degree{}", string)
	gsub("&middot; *", "\\ensuremath{*}", string)
	gsub("&times; *", "\\ensuremath{*}", string)
	gsub("&tau;", "\\ensuremath{\\tau}", string)
	gsub("&delta;", "\\ensuremath{\\delta}", string)
	gsub("&sigma;", "\\ensuremath{\\sigma}", string)
	gsub("&plusmn;", "\\ensuremath{\\pm}", string)
	gsub("&ldquo;", "``", string)
	gsub("&rdquo;", "''", string)
	gsub("&amp;", "\\&", string)
	gsub("&Aring;", "", string)
	gsub("&nbsp;", "~", string)
	gsub("&lt;", "\\ensuremath{<}", string)
	gsub("&gt;", "\\ensuremath{>}", string)
	string = gensub("&radic;\\(([^)]*)\\)",
	    	    	"\\\\ensuremath{\\\\sqrt{}{\\1}}", "g", string)
	gsub("&radic;", "\\ensuremath{\\sqrt{}{FIXME}}", string)
	gsub("[&]", "\\\\&{}", string)
	string = gensub("<e[0-9]>([^<]*)</e[0-9]>",
	    	    	    	"\\\\emph{\\1}", "g", string)
	string = gensub("<sup>TM</sup>",
	    	    	    	"\\\\texttrademark{}", "g", string)
	string = gensub("<sup>([^<]*)</sup>",
	    	    	    	"\\\\ensuremath{^{\\1}}", "g", string)
	string = gensub("<sub>([^<]*)</sub>",
	    	    	    	"\\\\ensuremath{_{\\1}}", "g", string)
	return string
}
