# fix-endnote-bibtex.awk
#
# Fix up BibTeX output produced by Endnote.
#
# Requires:
# * escape_tex.awk
# * ieee_names.awk
#
# Notes:
# * Some operations of this filter may only apply to the particular version of
#   the BibTeX export filter for endnote which I have used. You can use my copy
#   of it if you like.
# * The list of languages and acronyms, which are handled specially, is limited.
# * Limits of escape_tex.awk and ieee_names.awk apply.
#
# Programmed with gawk 3.1.1
#
# Copyright (C) by Volker Kuhlmann <VolkerKuhlmann@gmx.de>
# Released under the terms of the GNU General Public License (GPL) Version 2.
# See http://www.gnu.org/ for details.
#
# Volker Kuhlmann
#  16, 18, 20, 28 Feb; 21 Mar 2003
#

## Fix some of the acronyms (prevent lower-casing)
function fix_acronyms() {
    gsub("Ieee", "{IEEE}")
    gsub("Gps", "{GPS}")
    gsub("Dcf77", "{DCF77}")
    gsub("Ptb", "{PTB}")
    gsub("Utc", "{UTC}")
    gsub("Intelsat", "{INTELSAT}")
    gsub("Glonass", "{GLONASS}")
    gsub("Isbn", "{ISBN}")
    gsub("Iete", "{IETE}")
}

## Substitute some of the journal names with macros
function subst_journal_names(                  arr) {
    arr["new"] = $0
    correct_journal_names(arr, "new")
    arr["old"] = arr["new"]
    translate_journal_names(arr, "new")
    $0 = arr["new"]
    if (arr["new"] != arr["old"]) {
    	$0 = gensub("^([^{]*){(.*)}([^}]*)$", "\\1\\2\\3", $0)
    }
}

## Rename some of the fields to the names BibTeX uses
function rename_fields() {
    gsub(" *PlacePublished ", "address ")
    gsub(" *SecondaryTitle ", "booktitle ")
    gsub(" *Custom1 ", "annote ")
    gsub(" *Holdings ", "library ")
}


BEGIN {
    # remove whitespace and CR from beginning and end of line
    RS="[ \t]*\r*[ \t]*\n\r*[ \t]*"
    ORS="\n"
    
    # this character (or string) is added between keywords
    keyword_sep_char=";"
    if (ENVIRON["KEYWORD_SEP_CHAR"] != "") \
    	keyword_sep_char=ENVIRON["KEYWORD_SEP_CHAR"]
}

## Fix page numbers to use proper TeX number range syntax
{
    $0 = gensub("([ \\t]*[Pp]ages[ \\t]*=[ \\t]*{[ \\t]*[0-9/]*)-([^-].*)", \
    	    	"\\1--\\2", "")
}

## Fix keywords
$0 ~ /^ *[Kk]ey[Ww]ords *= *{[^}]*$/ {
    addchar=1
    print $0 keyword_sep_char
    next
}
addchar && $0 !~ /} *, *$/ {
    print $0 keyword_sep_char
    next
}
{   addchar=0 }

## Ditch some of the crap lines before renaming fields
$0 ~ /PlacePublished = {Place Published},/ { next }
$0 ~ /Custom1 = {Custom 1},/ { next }

## Try and deal with language
# (Drop entry type if "Article")
$0 ~ /^Note = {/ {
    line = $0
    $0 = gensub("Note = {(English|German|Chinese|French|Spanish|Italian|Croatian|Japanese)",
    	    	"language = {\\1},", "")
    if ($0 != line) waslanguage=2
}
waslanguage {
    if ($0 == "Article") next
    if (waslanguage == 1) {
    	$0 = "Note = {" $0
	if (! match($0, "},$")) $0 = $0 "."
    }
    waslanguage--
}

{
    subst_journal_names()   # must be before fix_acronyms()
    fix_acronyms()
    rename_fields()
    $0 = translate_html($0)
    print
}
