# ieeeris_to_bibtex.awk
#
# Convert citations exported from IEEE (they call the format "ISI Researchsoft")
# to BibTeX. The format is generated by http://ieeexplore.ieee.org/ in Feb 2003.
# The format is similar, but not identical, to RIS.
#
# Requires:
# * escape_tex.awk
# * ieee_names.awk
#
# Notes:
# * run your input through the IEEE-RIS-fixer first
#   (Many of the conference records exported by IEEE are faulty!)
# * bibtool is very useful for pretty-printing BibTeX.
# * On a Duron 600MHz, this converts appr 18 entries per second if they're in
#   separate files (and gawk is started up for each one), and appr 150 entries
#   per second if they're all in one file and gawk is started once.
# * If gawk is run per entry (via bib-fixer), the filenames will be added to the
#   library field (as \url{..}) of the BibTeX entry.
#
# Programmed with gawk 3.1.1
#
# Copyright (C) by Volker Kuhlmann <VolkerKuhlmann@gmx.de>
# Released under the terms of the GNU General Public License (GPL) Version 2.
# See http://www.gnu.org/ for details.
#
# Volker Kuhlmann
#  16, 17, 18, 19, 20, 21, 28 Feb; 1, 2, 6, 13, 21, 26 Mar; 5, 28, 29 Apr 2003
#

## Print entry
function print_entry(entrytype, entry,        field, numfields) {
    # Combine start and end page
    if ( entrytype == "" ) entrytype=miscname
    if ( entry["endpage"] == "" )
    	entry["pages"] = entry["startpage"]
    else
	entry["pages"] = entry["startpage"]"--"entry["endpage"]
    delete entry["startpage"]
    delete entry["endpage"]
    
    # Fix up broken output of IEEE standards
    if ( index(entry["journal"], "IEEE Std") ) {
    	entrytype=bookname
	entry["note"]=entry["journal"]
	entry["journal"]=""
	entry["ISBN"]=entry["ISSN"]
    }
    del_fields_if_empty(entry, "journal")
    
    # don't output empty number field for books
    if ( entrytype == bookname ) del_fields_if_empty(entry, "number ISSN")
    
    # don't output a single semicolon for keywords
    if ( entry["keywords"] == ";" ) entry["keywords"]=""

    # Escape TeX special chars
    escape_tex(entry, "title booktitle abstract keywords")
    
    # Deal with title fields, pages and author
    rearrange_authors(entry, "author")
    escape_title(entry, "title booktitle")
    del_fields_if_empty(entry, "pages author title booktitle abstract keywords")
    
    # Use BibTeX strings whenever possible
    translate_journal_names(entry, "journal")
    translate_conf_names(entry, "booktitle")
    del_fields_if_empty(entry, "journal booktitle")
    
    # Count number of fields
    numfields=0; for (field in entry) numfields++
    
    # Add filename if known (where the corresponding paper is stored)
    if (numfields > 0 && ENVIRON["CURRENT_FILENAME"] != "") {
    	#addtofield(entry, "url", ENVIRON["CURRENT_FILENAME"])
    	#addtofield(entry, "annote", "\\url{" ENVIRON["CURRENT_FILENAME"] "}")
    	addtofield(entry, "library", "\\url{" ENVIRON["CURRENT_FILENAME"] "}")
	sub(".ris}$", ".pdf}", entry["library"])
    }
    
    # Output nothing if entry is empty
    if (numfields == 0) return
    
    # Add the info that data for this entry came from IEEEXplore
    entry["database"] = "ieeexplore.ieee.org"
    
    # Generate output
    printf "@%s{citekey,\n", entrytype
    for (field in entry) {
    	# bad hack with journal names
    	if (field == "journal" && match(entry[field], "^IEE(E|)_[A-Z_]*$"))
	    printf "  %s=%s,\n", field, entry[field];
    	else 
	    #printf "  %s={%s},\n", field, entry[field]
    	    printf "  %s=%s,\n", field, \
	    	gensub("^{([0-9][0-9]*)}$", "\\1", 1, "{" entry[field] "}")
    }
    print "}"
    
    # Tidy up - or will cause bogus entries to be written on empty input lines
    delete ENVIRON["CURRENT_FILENAME"]
}

## Reset variables at each new entry
function init_entry() {
    delete entry
    delete doublecheck
    entrytype=""
    #if (ENVIRON["CURRENT_FILENAME"] != "")
    #	entry["url"] = ENVIRON["CURRENT_FILENAME"]
}


BEGIN {
    # remove whitespace and CR from beginning and end of line
    RS="[ \t]*\r*[ \t]*\n\r*[ \t]*"
    ORS="\n"
    
    inprocname = "InProceedings"
    articlename = "Article"
    bookname = "Book"
    miscname = "Misc"
    
    init_entry()
}

## remove trailing white space from the input
# (perhaps we could also do this by extending RS)
#{ sub("[ \t]*$", "") }

## ignore empty lines for now (must be changed for multiple entries per file)
$0 ~ /^[ \t]*$/ { next }

## preset variables
{
    line = substr ($0, 7)
    type = tolower( substr(line, 1, 4) )
    field=""
}

## entry type is first
$0 ~ /^TY *- */ && type == "jour" { entrytype=articlename }
$0 ~ /^TY *- */ && type == "conf" { entrytype=inprocname }
$0 ~ /^TY *- */ && type == "book" { entrytype=bookname }  # this doesn't occur
$0 ~ /^TY *- */ && type == "std" {
    entrytype=miscname
    addtofield(entry, "note", "IEEE standard (or IEE paper---doh)", ". ")
}

## entry fields
# Lines which are neither one of these fields nor a continuation thereof are
# ignored.
$0 ~ /^JO *- */ { field="journal" } 	# sometimes JA? as well?
$0 ~ /^IS *- */ { field="ISSN" }
$0 ~ /^PY *- */ { field="year" }
$0 ~ /^VO *- */ { field="volume" }  	# sometimes VL? as well?
$0 ~ /^VL *- */ { field="volume" }  	# sometimes only VL!
$0 ~ /^SN *- */ { field="number" }
$0 ~ /^TI *- */ { field="title" }
$0 ~ /^AB *- */ { field="abstract" }
$0 ~ /^SP *- */ { field="startpage" }
$0 ~ /^EP *- */ { field="endpage" }
$0 ~ /^KW *- */ { field="keywords" }
$0 ~ /^AU *- */ { field="author" }
#{ line = delspc(line); if ( line == "" ) next; }
field != "" {
    # check if the journal name needs to go into a different field
    if (field == "journal" && entrytype == inprocname) {
    	field = "booktitle" 	# change field name
    }
    # remove leading and trailing white space from value
    line = delspc(line)
    # check for second occurance of equivalent field
    if ( field == "volume" ) {
    	if ( doublecheck[field] \
	     &&  ( line == entry[field]  ||  line == "" ) ) {
	    lastfield = field
	    next
	}
	doublecheck[field] = 1
    }
    # add current line to field
    addtofield(entry, field, line)
    lastfield = field
    next
}

$0 !~ /^[A-Z][A-Z0-9] *- */ {
    addtofield(entry, lastfield, $0)
}

## End of record - there could be further records in the input
$0 ~ /^ER *- */ {
    print_entry(entrytype, entry)
    init_entry()
}

END {
    print_entry(entrytype, entry)
}
